#include <QtGui>
#include <QtCore>
#include <QtXml/QDomDocument>
#include <QFile>

#include "sp.h"
#include "ui_sp.h"

SP::SP(QWidget *parent)
    : QMainWindow(parent)
{
    ui.setupUi(this);
    QStringList headerLabels;
    headerLabels << tr("Smile") << tr("File");
    ui.treeWidget->setHeaderLabels(headerLabels);
    ui.addr->setHidden(1);

    ui.treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
}

void SP::on_actionOpen_triggered()
{
    filePath = QFileDialog::getOpenFileName(this, tr("Open File"), getenv( "HOME" ), tr("XML files (emoticons.xml);;All files (*);;"));

    workDir=filePath;
    workDir.truncate(filePath.lastIndexOf("/"));
    ui.statusBar->showMessage(tr("Reading: ")+filePath);
    loadFile(filePath);

}

void SP::loadFile(const QString &filePath)
{

    if (!filePath.isEmpty()){
        ui.treeWidget->clear();
        QFile file(filePath);
        if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
            //Устанавливаем поток
            QTextStream in(&file);
            //Кодировку для потока
            in.setCodec("UTF-8");
            xml.clear();
            xml.addData(in.readAll());
            parseXml();
            file.close();
            ui.addr->setHidden(0);
            ui.addr->setText(filePath);
            ui.statusBar->showMessage(tr("Ready!"),5000);
        }else{
            ui.statusBar->showMessage(tr("Unable to open!"),5000);
            ui.addr->setHidden(1);

        }
    }else{
        ui.statusBar->showMessage(tr("File not selected!"),5000);
    }
}


void SP::parseXml()
{
    QList<QTreeWidgetItem *> tt;
    ui.treeWidget->clear();

    while (!xml.atEnd()) {
        xml.readNext();
        if (xml.isStartElement()) {
            if (xml.name() == "emoticon") fileString = xml.attributes().value("file").toString();
            currentTag = xml.name().toString();
        } else if (xml.isEndElement()) {
            if (xml.name() == "emoticon") {
                QTreeWidgetItem *item = new QTreeWidgetItem;
                item->setText(1, fileString);
                item->setIcon(0,QIcon(QPixmap(workDir+"/"+fileString)));
                item->addChildren(tt);
                item->setIcon(2,QIcon(QPixmap(":/cut/edit-cut.png")));
                ui.treeWidget->addTopLevelItem(item);
                fileString.clear();
            }

        } else if (xml.isCharacters() && !xml.isWhitespace()) {
            if (currentTag == "string"){
                QTreeWidgetItem *item = new QTreeWidgetItem;
                item->setText(0, xml.text().toString());
                tt+=item;
            }
        }
    }
    if (xml.error() && xml.error() != QXmlStreamReader::PrematureEndOfDocumentError) {
        qWarning() << "XML ERROR:" << xml.lineNumber() << ": " << xml.errorString();
    }
}


SP::~SP()
{}

void SP::on_actionQuit_triggered()
{
    close();
}

void SP::on_actionQutIMSP_triggered()
{
    QMessageBox::about(this,tr("About QutIM Smile Packer"),tr("QutIM Smile Packer developed & (C) 2009 by DEXif(<a href='http://dexif.ExGraphics.info'>DEXif.ExGraphics.info</a>)\n License: GNU General Public License Version 2"));
}

void SP::on_treeWidget_itemClicked(QTreeWidgetItem* item, int column)
{
    if(!item->text(1).isEmpty()){
        currentItem=item;
    }else if(!item->parent()->text(1).isEmpty()){
        currentItem=item->parent();
    }
    QMovie *movie = new QMovie(workDir+"/"+currentItem->text(1));
    ui.Filename->setText(currentItem->text(1));
    ui.img->setMovie(movie);
    movie->start();
    QString str;
    str.clear();
    for(int i=0;i<currentItem->childCount();i++){
        str+=currentItem->child(i)->text(0);
        if(i!=currentItem->childCount()-1)str+="\n";
    }
    ui.plainTextEdit->setPlainText(str);



}

void SP::on_actionSave_triggered()
{
    QTreeWidgetItem *item = ui.treeWidget->invisibleRootItem();
    QString tmp;
    QString rpl;
    tmp.clear();
    tmp="<?xml version=\"1.0\"?>\n<messaging-emoticon-map>\n";
    for (int i = 0; i < item->childCount(); i++){
        tmp+="\t<emoticon file=\""+item->child(i)->text(1)+"\">\n";

        for (int j=0;j<item->child(i)->childCount();j++){
            rpl.clear();
            rpl = item->child(i)->child(j)->text(0);
            rpl.replace("&", "&amp;");
            rpl.replace("<", "&lt;");
            rpl.replace(">", "&gt;");
            rpl.replace("'", "&apos;");
            rpl.replace("\"", "&quot;");
            tmp+="\t\t<string>"+rpl+"</string>\n";
        }
        tmp+="\t</emoticon>\n";
    }
    tmp+="</messaging-emoticon-map>\n";
    int s = QMessageBox::question(this, tr("Rewrite file?"),tr("Do you want to rewrite this smilepack?"),QMessageBox::Yes | QMessageBox::No | QMessageBox::Escape);
    if(s == QMessageBox::Yes) {QFile file(filePath);
        file.open(QIODevice::WriteOnly | QIODevice::Text);
        file.write(tmp.toUtf8());
        file.close();
        ui.statusBar->showMessage(tr("Saved!"),5000);
    }
}

void SP::deleteitem(){


         QTreeWidgetItem *parent = ui.treeWidget->currentItem()->parent();
         int index;
         if (parent) {
             index = parent->indexOfChild(ui.treeWidget->currentItem());
             delete parent->takeChild(index);
         } else {
             index = ui.treeWidget->indexOfTopLevelItem(ui.treeWidget->currentItem());
             delete ui.treeWidget->takeTopLevelItem(index);
         }
         ui.statusBar->showMessage(tr("Deleted!"),5000);
}


void SP::additem(){
    bool ok;
    QString smile = QInputDialog::getText(this, tr("Enter associative string"), tr("Smile:"), QLineEdit::Normal,"", &ok);
    if (ok && !smile.isEmpty()){
        QTreeWidgetItem *parent = ui.treeWidget->currentItem();
        QTreeWidgetItem *newItem;
        if (parent){
            newItem = new QTreeWidgetItem(parent, ui.treeWidget->currentItem());
            newItem->setText(0,smile);
            ui.statusBar->showMessage("Smile added: "+smile,5000);
        }
    }
}

void SP::on_treeWidget_customContextMenuRequested(QPoint pos)
{
        if (ui.treeWidget->topLevelItemCount()>0) {
            QMenu menu(ui.treeWidget);
            QAction *del=menu.addAction("Delete Row");
            QObject::connect(del, SIGNAL(triggered()), this , SLOT(deleteitem()));
            if(!ui.treeWidget->currentItem()->text(1).isEmpty()){
                QAction *add=menu.addAction("Add Row");
                QObject::connect(add, SIGNAL(triggered()), this , SLOT(additem()));
            }
            menu.exec(QCursor::pos());
        }
}

void SP::on_actionNew_triggered()
{
    workDir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),getenv( "HOME" ),QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
    filePath = workDir+"/emoticons.xml";
    QFile file(filePath);
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)){
        file.close();
        ui.statusBar->showMessage(tr("Reading: ")+filePath);
        loadFile(filePath);
    }
}

void SP::on_treeWidget_doubleClicked(QModelIndex index)
{
    if(ui.treeWidget->currentItem()->text(1).isEmpty()){
            bool ok;
            QString smile = QInputDialog::getText(this, tr("Enter associative string"), tr("Smile:"), QLineEdit::Normal,ui.treeWidget->currentItem()->text(0), &ok);
            if (ok && !smile.isEmpty()) ui.treeWidget->currentItem()->setText(0,smile);
    }
}
